@@ -19,9 +19,8 @@ pal.col <- RColorBrewer::brewer.pal(npal, pal)
 cols <- rev(grDevices::colorRampPalette(pal.col)(maxx))
 
 to_df <- function(W){
-	# convert parabolic results to data.frame
-	with(W, data.frame(taper_seq=as.vector(taper_seq), 
-					   taper_weights=as.vector(taper_weights)))
+  # convert parabolic results to data.frame
+  with(W, data.frame(taper_seq=as.vector(taper_seq), taper_weights=as.vector(taper_weights)))
 }
 
 ## a roundabout way of bootstrapping y-axis limits:
@@ -31,7 +30,7 @@ DfU <- to_df(WgtsU)
 #  lower
 WgtsL <- parabolic_weights_fast(maxx)
 DfL <- to_df(WgtsL)
-#
+
 ylims <- range(pretty(dB(c(DfL$taper_weights, DfU$taper_weights)))) + c(-2,5)
 
 # function for plotting text
@@ -42,7 +41,7 @@ TFUN <- function(Df.){
 }
 
 # function for weighting factors and plotting
-WFUN<-function(x){
+WFUN <- function(x){
   message(x)
   Wgts <- parabolic_weights_fast(x)
   Df <- to_df(Wgts)
@@ -53,8 +52,8 @@ WFUN<-function(x){
 
 ## Plot parabolic weighting, in dB, colored by maximum num tapers
 plot(dB(taper_weights) ~ log10(taper_seq), DfU, type="s", 
-	 xlim=c(0, log10(maxx)+0.2), 
-	 ylim=ylims, yaxs="i",
+     xlim=c(0, log10(maxx)+0.2), 
+     ylim=ylims, yaxs="i",
      col=cols[5], lwd=2,  
      main="Multitaper weighting factors by maximum tapers applied",
      xlab="log10 taper sequence", 
